//basic GLUT text rendering

#include <GL/glut.h> //glut library, it imports gl.h, glu.h
#include <stdlib.h> //for exit call
#include <stdio.h>
#include <iostream>

#define ESCAPE 27 //define ASCII value for escape keystroke

#define RABBIT 1;
#define HOUND 2;

int SELECT = RABBIT; //the default text displayed is that of the rabbit

int window_width = 500; //sets the width of the window
int window_height = 100; //sets the hight of the window

/*
=================
switchToOrth
-switches from perspective to orthographic projection
NOTE: (0,0) is situated at the top left corner of your screen
=================
*/
void switchToOrtho() 
{
	/* specifies the Projection matrix as the current matrix */
	glMatrixMode(GL_PROJECTION);
	/* pushes current matrix stack down by one */
	glPushMatrix();
	/* move to the center of the screen */
	glLoadIdentity();
	/* define a 2-D orthographic projection matrix */
	gluOrtho2D(0, window_width, 0, window_height);
	/* scales along the x, -y, and z axes */
	glScalef(1, -1, 1);
	/* moves along the x, y and z-axis, moves -'window_hight' units (down) on y-axis */
	glTranslatef(0, -1.0*window_height, 0);
	/* specifies the Modelview matrix as the current matrix */
	glMatrixMode(GL_MODELVIEW);
	/* move to the center of the screen */
	glLoadIdentity();
}

/*
=================
switchFromOrth
-switches back from orthographic projection to perspective projection
=================
*/
void switchFromOrtho() 
{
	/* specifies the Projection matrix as the current matrix */
	glMatrixMode(GL_PROJECTION);
	/* pops current matrix stack */
	glPopMatrix();
	/* specifies the Modelview matrix as the current matrix */
	glMatrixMode(GL_MODELVIEW);
}

/*
=================
display
-renders output/scene to the screen
=================
*/
void display(void) 
{
	/* clear frame buffer and z-buffer */
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	/* swith to orthographic projection, we are currently in perspective projection,
	   the text has to be rendered in orthographic projection mode */
	switchToOrtho();

	/* set text colour to white */
   	glColor3f(1.0,1.0,1.0);
	/* sets the font position roughly in the middle of the window */
	glRasterPos2i(window_width/2 - 150, window_height/2);
	
	/* draws the text */
	if(SELECT == 1) //if select equals RABBIT
	{
		char string[] = "The snow white rabbit leaped into the hole";
		for(int i = 0; string[i]; i++)
			glutBitmapCharacter(GLUT_BITMAP_8_BY_13, string[i]);
	}
	else
	{
		char string[] = "The hairy hound followed close behind";
		for(int i = 0; string[i]; i++)
			glutBitmapCharacter(GLUT_BITMAP_8_BY_13, string[i]);
	}

	/* switch back to perspective projection */
	switchFromOrtho();

	/* make sure all previously issued commands are completed */
	glFlush(); 
	/* contents of back buffer are set to become that of front buffer (double buffering)*/
	glutSwapBuffers();
}

/*
=================
controlMenu
-Used for the right-click accessible mouse menu
=================
*/
void controlMenu(int x)
{
	/* basic switch control structure, prints the text corresponding with the selected
	   action in the command prompt window */
	switch(x)
	{
		case 1: SELECT = RABBIT;
			break;
		case 2: SELECT = HOUND;;
			break;
		case 3: exit(0); //terminates the application
	}
	/* sets the current window for redisplay */
	glutPostRedisplay();
}

/*
=================
menu
-The mouse menu GLUT routines
=================
*/
void menu()
{
	/* creates a new pop-up menu */
	glutCreateMenu(controlMenu);
	/* adds menu entries to the bottom of the current menu */
	glutAddMenuEntry("The snow white rabbit...",1);
	glutAddMenuEntry("The hairy hound...",2);
	glutAddMenuEntry("Quit",3);
	/* attaches the right mouse button for menu selection */
	glutAttachMenu(GLUT_RIGHT_BUTTON);
}

/*
=================
main
=================
*/
int main(int argc, char** argv)
{
	/* initialize the GLUT library. */
	glutInit(&argc, argv);
	/* sets the initial display mode: double buffering and a depth buffer. */
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
	/* sets default window size */
	glutInitWindowSize(window_width, window_height);
	/* creates a top-level window with a caption */
	glutCreateWindow("...displaying text...");
	/* displays our contents in the window */
	glutDisplayFunc(display);
	/* enable hidden-surface removal in our case hiding the floor behind 
	   the wall facing the viewer - remove it to see the result */
	glEnable(GL_DEPTH_TEST);
	/* calls our menu function */
	menu();
	/* enters the GLUT event processing loop - will handle any callback */
	glutMainLoop();
	return 0;
}
//EOF