//Cg vertex lighting example

#include <GL/glut.h>
#include <CG/cg.h>
#include <CG/cgGL.h>

#include <iostream>
#include <math.h>
#include <string.h>
#include <time.h>

using namespace std;

#define ESCAPE 27
#define PAGE_UP 73
#define PAGE_DOWN 81
#define UP_ARROW 72
#define DOWN_ARROW 80
#define LEFT_ARROW 75
#define RIGHT_ARROW 77
#define BALL_TEXTURE 0

const double PI = 3.14159;

static int GLUT_window = 0;
static double cam_pos[6] = {0.1, 4.0, 6.0, 0.0, 0.0, 0.0};
float rotation_angle = 0.0;

int window_windows_width = 800;
int window_window_height = 600;

/* Cg globals */
CGcontext cg_context;
CGprogram vertex_program;
CGprogram fragment_program;
CGprofile cgVertexProfile = CG_PROFILE_UNKNOWN;
CGprofile cgFragmentProfile = CG_PROFILE_UNKNOWN;

GLUquadricObj *ball_object = NULL;

struct Image
{
    unsigned long size_x;
    unsigned long size_y;
    char *data;
};

const int texture_number = 1;
static GLuint textureName;

Image BALL_TEXTURE_DATA[texture_number];
GLuint mainTexture[texture_number+1];

char* textureNames[texture_number] = {"texture.bmp"};

/* loads 24-bit bitmap files with 1 plane only */
/* (Disclaimer: This function originally obtained from http://nehe.gamedev.net) */
int ImageLoad(const char *filename, Image *image) {

    FILE *file;
    unsigned long size;
    unsigned long i;
    unsigned short int planes;
    unsigned short int bpp;
    char temp, finalName[80];

	strcpy(finalName, "" );
	strcat(finalName, filename );

    if ((file = fopen(finalName, "rb"))==NULL) {
		printf("File Not Found : %s\n",finalName);
		return 0;
    }

    fseek(file, 18, SEEK_CUR);

    if ((i = fread(&image->size_x, 4, 1, file)) != 1) {
		printf("Error reading windows_width from %s.\n", finalName);
		return 0;
    }

    if ((i = fread(&image->size_y, 4, 1, file)) != 1) {
		printf("Error reading window_height from %s.\n", finalName);
		return 0;
    }

    size = image->size_x * image->size_y * 3;

    if ((fread(&planes, 2, 1, file)) != 1) {
		printf("Error reading planes from %s.\n", finalName);
		return 0;
    }

    if (planes != 1) {
		printf("Planes from %s is not 1: %u\n", finalName, planes);
		return 0;
    }

    if ((i = fread(&bpp, 2, 1, file)) != 1) {
		printf("Error reading bpp from %s.\n", finalName);
		return 0;
    }

    if (bpp != 24) {
	printf("Bpp from %s is not 24: %u\n", finalName, bpp);
	return 0;
    }

    fseek(file, 24, SEEK_CUR);

    image->data = (char *) malloc(size);
    if (image->data == NULL) {
		printf("Error allocating memory for color-corrected image data");
		return 0;
    }

    if ((i = fread(image->data, size, 1, file)) != 1) {
		printf("Error reading image data from %s.\n", finalName);
		return 0;
    }

    for (i=0;i<size;i+=3) {
		temp = image->data[i];
		image->data[i] = image->data[i+2];
		image->data[i+2] = temp;
    }

    return 1;
}

/* load bitmap images and convert them to textures */
void GLTextureLoader()
{
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);

	for (int k=0; k < texture_number; k++)
	{
		if (!ImageLoad(textureNames[k], &BALL_TEXTURE_DATA[k]))
			exit(1);

		/* texture specific parameters */
		glGenTextures(1, &mainTexture[k]);
	    glBindTexture(GL_TEXTURE_2D, mainTexture[k]);
	   	glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
	   	glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
	   	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_NEAREST);
	   	gluBuild2DMipmaps(GL_TEXTURE_2D, 3, BALL_TEXTURE_DATA[k].size_x, BALL_TEXTURE_DATA[k].size_y, GL_RGB, GL_UNSIGNED_BYTE, BALL_TEXTURE_DATA[k].data);
	}

	textureName = mainTexture[0];

}

/* keyboard input processing */
void keyHandler(unsigned char key, int x, int y)
{
	float view_distance = 12.0;

	if (key == 'a')
	{
		rotation_angle += 3.5;
		cam_pos[0] = cos(rotation_angle * PI / 180.0)*view_distance;
		cam_pos[2] = sin(rotation_angle * PI / 180.0)*view_distance;
	}

	if (key == 'd')
	{
		rotation_angle -= 3.5;
		cam_pos[0] = cos(rotation_angle * PI / 180.0)*view_distance;
		cam_pos[2] = sin(rotation_angle * PI / 180.0)*view_distance;
	}

	if (key == 'w')
	{
		cam_pos[1] += 0.5;
	}

	if (key == 's')
	{
		cam_pos[1] -= 0.5;
	}

	if (key == ESCAPE)
	{
		exit(0);
	}

	glutPostRedisplay();
}

/* handling of pressed keys */
void keysPressed(int key, int x, int y)
{
    switch (key) {
    	case GLUT_KEY_PAGE_UP:
			keyHandler('z',0,0);
			break;

		case GLUT_KEY_DOWN:
			keyHandler('s',0,0);
			break;

		case GLUT_KEY_LEFT:
			keyHandler('a',0,0);
			break;

		case GLUT_KEY_RIGHT:
			keyHandler('d',0,0);
			break;

    	case GLUT_KEY_PAGE_DOWN:
			keyHandler('x',0,0);
			break;

	    case GLUT_KEY_UP:
			keyHandler('w',0,0);
			break;
	}
}

// Release resources
void cleanup()
{
	cgDestroyProgram(vertex_program);
	cgDestroyProgram(fragment_program);
	cgDestroyContext(cg_context);
}

void bindCgVariables()
{
	cgGLSetStateMatrixParameter(cgGetNamedParameter(vertex_program, "modelviewProjection"), CG_GL_MODELVIEW_PROJECTION_MATRIX, CG_GL_MATRIX_IDENTITY);
	cgGLSetParameter3f(cgGetNamedParameter(vertex_program, "objectspaceLightPosition"), 0.0, 10.0, 10.0);
	cgGLSetParameter3f(cgGetNamedParameter(vertex_program, "objectspaceEyePosition"), cam_pos[0], cam_pos[1], cam_pos[2]);
}

/* reshape function */
void reshape(GLsizei windows_width, GLsizei window_height)
{
	if (window_height == 0)
		window_height = 1;

	glViewport(0, 0, windows_width, window_height);

	glMatrixMode(GL_PROJECTION);
    glLoadIdentity();

	gluPerspective(45.0, (GLfloat)windows_width/window_height, 0.1, 3000.0);

	glMatrixMode(GL_MODELVIEW);

	glLoadIdentity();
}

// Sets up some global OpenGL parameters..
void setupScene(int argc, char **argv)
{
	GLTextureLoader();

	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);

	glShadeModel(GL_SMOOTH);
	glClearDepth(1.0f);

	ball_object = gluNewQuadric();

	gluQuadricNormals(ball_object, GLU_SMOOTH);
	gluQuadricOrientation(ball_object, GLU_OUTSIDE);
	gluQuadricTexture(ball_object, GL_TRUE);
}

/* the main rendering function */
void render()
{
	glutSetWindow(GLUT_window);
	glClearColor(1.0, 0.8, 1.0, 1.0);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	/* init camera position */
	gluLookAt(cam_pos[0],cam_pos[1],cam_pos[2],cam_pos[3],cam_pos[4],cam_pos[5], 0.0, 1.0, 0.0);

	bindCgVariables();
	gluSphere(ball_object, 4, 36, 14);

	glutSwapBuffers();

}

/* setup the Cg variables */
void CgVarSetup()
{
	/* create the Cg context */
	cg_context = cgCreateContext();

	/* determine the vertex and fragment shader profiles */
	cgVertexProfile = cgGLGetLatestProfile(CG_GL_VERTEX);
	cgFragmentProfile = cgGLGetLatestProfile(CG_GL_FRAGMENT);

	/* set the vertex and fragment shader profiles */
	cgGLSetOptimalOptions(cgVertexProfile);
	cgGLSetOptimalOptions(cgFragmentProfile);

	/* access the Cg source file */
	vertex_program = cgCreateProgramFromFile(cg_context, CG_SOURCE, "vlight_vprog.cg", cgVertexProfile, "main", 0);
	fragment_program = cgCreateProgramFromFile(cg_context, CG_SOURCE, "vlight_fprog.cg", cgFragmentProfile, "main", 0);

	/* load, enable and bind the Cg vertex program */
	cgGLLoadProgram(vertex_program);
	cgGLEnableProfile(cgVertexProfile);
	cgGLBindProgram(vertex_program);

	/* load, enable and bind the Cg fragment program */
	cgGLLoadProgram(fragment_program);
	cgGLEnableProfile(cgFragmentProfile);
	cgGLBindProgram(fragment_program);

}

/* set the Cg effect */
void setupCgEffects()
{
	cgGLSetParameter3f(cgGetNamedParameter(vertex_program, "incomingAmbientLightColour"), 0.5, 0.5, 0.5);
	cgGLSetParameter3f(cgGetNamedParameter(vertex_program, "incomingDiffuseLightColour"), 1.0, 1.0, 1.0);

	cgGLSetParameter3f(cgGetNamedParameter(vertex_program, "materialAmbientReflectance"), 0.1, 0.1, 0.1);
	cgGLSetParameter3f(cgGetNamedParameter(vertex_program, "materialDiffuseColour"), 0.5, 0.5, 0.5);
	cgGLSetParameter3f(cgGetNamedParameter(vertex_program, "materialSpecularColour"), 1.0, 1.0, 1.0);
	cgGLSetParameter1f(cgGetNamedParameter(vertex_program, "meterialShininess"), 100.0);

	cgGLSetTextureParameter(cgGetNamedParameter(fragment_program, "textureMap"), textureName);
	cgGLEnableTextureParameter(cgGetNamedParameter(fragment_program, "textureMap"));
}

/* application entry point */
int main(int argc, char **argv)
{
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DEPTH  | GLUT_DOUBLE | GLUT_RGBA);
	glutInitWindowPosition(100,100);
	glutInitWindowSize(window_windows_width,window_window_height);

	GLUT_window = glutCreateWindow("Cg vertex lighting");
	glutReshapeFunc(reshape);
	glutDisplayFunc(render);
	glutKeyboardFunc(keyHandler);
	glutSpecialFunc(keysPressed);

	setupScene(argc,argv);
	CgVarSetup();

	keyHandler('a', 0, 0);
	setupCgEffects();
	bindCgVariables();

	glutMainLoop();

	cleanup();

	return 1;
}

